# Copyright (c) 2012, Christian Kampka
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import yaml

import fixtures
from testtools import TestCase, ExpectedException
from testtools.matchers import LessThan, HasLength

from yconf.config import _Loader as Loader, BaseConfiguration


class BaseYamlFileFixture(fixtures.Fixture):

    default_config = {"production": {"a": "a", "b": "b", "c": "c", "e" : { "f": "g" }},
                      "staging": {"b": "B"},
                      "development": {"c": "C"}
    }

    def __init__(self, data=None):
        super(BaseYamlFileFixture, self).__init__()
        self.data = data or self.default_config

    def setUp(self):
        super(BaseYamlFileFixture, self).setUp()
        self.dir = self.useFixture(fixtures.TempDir())


class YamlFileFixture(BaseYamlFileFixture):

    def __init__(self, data=None, name="config.yml"):
        super(YamlFileFixture, self).__init__(data)
        self.name = name

    def setUp(self):
        super(YamlFileFixture, self).setUp()
        self.config = os.path.join(self.dir.path, self.name)
        with open(self.config, "w") as f:
            f.write(yaml.dump(self.data))


class YamlConfigDirFixture(BaseYamlFileFixture):

    def setUp(self):
        super(YamlConfigDirFixture, self).setUp()
        for key, value in self.data.items():
            with open(os.path.join(self.dir.path, "%s.yml" % key), "w") as f:
                f.write(yaml.dump(value))


class LoaderTest(TestCase):

    def setUp(self):
        TestCase.setUp(self)
        self.data = {"testcase": {"test-level": 1}}

    def test_constructMapping(self):

        d = yaml.load(yaml.dump(self.data), Loader=Loader)

        self.assertTrue("test_level" in d["testcase"])
        self.assertEqual(d["testcase"]["test_level"],
                        self.data["testcase"]["test-level"])

    def test_throwOnMappingConflict(self):

        data = {"testcase": {"test-level": 1, "test_level": 2}}

        with ExpectedException(Exception, ".+'test_level'.+'test-level'.+"):
            yaml.load(yaml.dump(data), Loader=Loader)

class BaseConfigurationTest(TestCase):

    def test_getEnvironment(self):
        bc = BaseConfiguration()
        self.assertEqual(10, bc.getEnvironment(10))
        self.assertEqual(20, bc.getEnvironment(20))
        self.assertEqual(30, bc.getEnvironment(30))

        self.assertEqual(10, bc.getEnvironment("production"))
        self.assertEqual(20, bc.getEnvironment("staging"))
        self.assertEqual(30, bc.getEnvironment("development"))

        self.assertThat(bc.getEnvironment("production"),
                        LessThan(bc.getEnvironment("staging")))

        self.assertThat(bc.getEnvironment("staging"),
                        LessThan(bc.getEnvironment("development")))

    def test_mergeEnvironments(self):
        f = self.useFixture(YamlFileFixture())

        bc = BaseConfiguration()
        bc.parse(args=["-c", f.config])
        self.assertEqual("a", bc["a"])
        self.assertEqual("b", bc["b"])
        self.assertEqual("c", bc["c"])
        del bc

        bc = BaseConfiguration()
        bc.parse(args=["-c", f.config, "-e", "staging"])
        self.assertEqual("a", bc["a"])
        self.assertEqual("B", bc["b"])
        self.assertEqual("c", bc["c"])
        del bc

        bc = BaseConfiguration()
        bc.parse(args=["-c", f.config, "-e", "development"])
        self.assertEqual("a", bc["a"])
        self.assertEqual("B", bc["b"])
        self.assertEqual("C", bc["c"])
        del bc

    def test_noMergeEnvironments(self):
        f = self.useFixture(YamlFileFixture())

        bc = BaseConfiguration(merge=False)
        bc.parse(args=["-c", f.config])
        self.assertEqual("a", bc["a"])
        self.assertEqual("b", bc["b"])
        self.assertEqual("c", bc["c"])
        del bc

        bc = BaseConfiguration(merge=False)
        bc.parse(args=["-c", f.config, "-e", "staging"])
        self.assertFalse(bc.has("a"))
        self.assertEqual("B", bc["b"])
        self.assertFalse(bc.has("c"))
        del bc

        bc = BaseConfiguration(merge=False)
        bc.parse(args=["-c", f.config, "-e", "development"])
        self.assertFalse(bc.has("a"))
        self.assertFalse(bc.has("b"))
        self.assertEqual("C", bc["c"])

    def test_configDirectory(self):
        f = self.useFixture(YamlConfigDirFixture())

        bc = BaseConfiguration()
        bc.parse(args=["-c", f.dir.path])
        self.assertEqual("a", bc["a"])
        self.assertEqual("b", bc["b"])
        self.assertEqual("c", bc["c"])
        del bc

        bc = BaseConfiguration()
        bc.parse(args=["-c", f.dir.path, "-e", "staging"])
        self.assertEqual("a", bc["a"])
        self.assertEqual("B", bc["b"])
        self.assertEqual("c", bc["c"])
        del bc

        bc = BaseConfiguration()
        bc.parse(args=["-c", f.dir.path, "-e", "development"])
        self.assertEqual("a", bc["a"])
        self.assertEqual("B", bc["b"])
        self.assertEqual("C", bc["c"])

    def test_parse(self):

        class TestConfiguration(BaseConfiguration):

            def makeParser(_self):
                parser = super(TestConfiguration, _self).makeParser()
                parser.add_argument("-a", dest="a")
                parser.add_argument("-x", dest="x")
                return parser

        f = self.useFixture(YamlFileFixture())

        bc = TestConfiguration()
        bc.parse(args=["-c", f.config, "-e", "development", "-a", "1", "-x", "2"])
        self.assertEqual("1", bc["a"])
        self.assertEqual("B", bc["b"])
        self.assertEqual("C", bc["c"])
        self.assertEqual("2", bc["x"])

    def test_parse_default_args(self):

        class TestConfiguration(BaseConfiguration):

            def makeParser(_self):
                parser = super(TestConfiguration, _self).makeParser()
                parser.add_argument("-a", dest="a")
                parser.add_argument("-b", dest="b")
                parser.add_argument("-x", dest="x")
                return parser

        f = self.useFixture(YamlFileFixture())
        bc = TestConfiguration()
        bc.parse(args=["-c", f.config, "-e", "development", "-a", "1"])

        with ExpectedException(KeyError):
            bc["x"]

        self.assertEqual("1", bc["a"])
        self.assertEqual("B", bc["b"])

    def test_parseWithoutConfig(self):
        class TestConfiguration(BaseConfiguration):

            def makeParser(_self):
                parser = super(TestConfiguration, _self).makeParser()
                parser.add_argument("-a", dest="a")
                parser.add_argument("-x", dest="x")
                return parser

        bc = TestConfiguration()
        bc.parse(args=["-a", "1", "-x", "2"])
        self.assertEqual("1", bc["a"])
        self.assertEqual("2", bc["x"])

    def test_config_before_defaults(self):
        """
        In cases where an argument is unfulfilled but defaulted in the arg parser,
        the config file should take precedence over parser defaults.
        """
        f = self.useFixture(YamlConfigDirFixture())

        class TestConfiguration(BaseConfiguration):

            def makeParser(_self):
                parser = super(TestConfiguration, _self).makeParser()
                parser.add_argument("-a", dest="e.f", default="x")
                parser.add_argument("-b", dest="a", default="b")
                parser.add_argument("-x", dest="c.c", default="c")
                parser.add_argument("-y", dest="x", default="y")


                return parser

        bc = TestConfiguration()
        bc.parse(args=["-c", f.dir.path])
        self.assertEqual("a", bc.a)
        self.assertEqual("g", bc.e.f)
        self.assertEqual("c", bc.c.c)
        self.assertEqual("y", bc.x)


def test_suite():
    from unittest import TestLoader
    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2012, Christian Kampka
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import yaml
import argparse

from yconf.util import NestedDict


class _Loader(yaml.SafeLoader):

    def __init__(self, *args, **kwargs):
        super(_Loader, self).__init__(*args, **kwargs)

    def construct_mapping(self, node, deep=False):
        result = dict()
        mapping = super(_Loader, self).construct_mapping(node, deep)
        for key, value in mapping.items():
            result[key] = value
            if type(key) == str and "-" in key:
                print(mapping)
                new = key.replace("-", "_")
                if new in mapping:
                    raise Exception("Key '%s' causes a mapping conflict with key '%s'." % (new, key))
                result[new] = value
        return result


PRODUCTION  = 10
STAGING     = 20
DEVELOPMENT = 30

_environmentNames = {
    PRODUCTION      : "production",
    STAGING         : "staging",
    DEVELOPMENT     : "development",
    "production"    : PRODUCTION,
    "staging"       : STAGING,
    "development"   : DEVELOPMENT
    }


class BaseConfiguration(NestedDict):

    _environments = ("production", "staging", "development")

    def __init__(self, merge=True):

        NestedDict.__init__(self, {})

        self.merge = merge

        self.configPath = None
        self.environment = "production"
        self.parser = self.makeParser()

    def getEnvironment(self, environment):
        if isinstance(environment, int):
            return environment
        elif str(environment) == environment:
            if environment not in _environmentNames:
                raise ValueError("Unknown environment: %s", environment)
            return _environmentNames[environment]
        else:
            raise TypeError("Environment value is not an integer or a valid string: %r" % environment)

    def makeParser(self):
        self._configParser = argparse.ArgumentParser(add_help=False)
        self._configParser.add_argument("-c", "--config", dest="configPath",
                                        help="Configuration file or directory containing the configuration files.")
        self._configParser.add_argument("-e", "--environment", default="production",
                                        choices=("production", "staging", "development"),
                                        help="The environment used for configuration. (default: production)")

        parser = argparse.ArgumentParser(parents=[self._configParser], argument_default=argparse.SUPPRESS)
        return parser

    def parse(self, args):

        args, remaining_argv = self._configParser.parse_known_args(args=args, namespace=self)
        if self.configPath:
            self.loadConfig()
        self.parser.parse_args(remaining_argv, self)

    def loadConfig(self):
        d = {}

        path = os.path.abspath(self.configPath)
        if os.path.isfile(path):
            with open(path, "r") as f:
                d = yaml.load(f.read(), Loader=_Loader)
        elif os.path.isdir(self.configPath):
            for e in self._environments:
                if os.path.exists(os.path.join(path, "%s.yml" % e)):
                    with open(os.path.join(path, "%s.yml" % e), "r") as f:
                        d[e] = yaml.load(f.read(), Loader=_Loader)
        if self.merge:
            for e in self._environments:
                if e in d and self.getEnvironment(e) <= self.getEnvironment(self.environment):
                    self.update(d[e])
        else:
            self.update(d.get(self.environment, {}))


__all__ = ["BaseConfiguration"]



# Copyright (c) 2012, Christian Kampka
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from testtools import TestCase
from testtools.matchers import IsInstance

from yconf.util import NestedDict


class ConfigEntryTest(TestCase):

    def setUp(self):
        TestCase.setUp(self)
        self.data = {"a": {"b": {"c": "d"}, "e": "f"}}

    def nestedToDict(self, nested):
        if type(nested) not in (NestedDict, dict):
            self.fail("Unexprected type %s, expected NestedDict" % type(nested))

        result = {}
        for (key, value) in nested.items():
            if type(value) is NestedDict:
                result[key] = self.nestedToDict(value)
            else:
                result[key] = value
        return result

    def test_dictAccess(self):

        nd = NestedDict(self.data)

        self.assertThat(nd["a"], IsInstance(NestedDict))
        self.assertEqual(self.nestedToDict(nd["a"].data), self.data["a"])
        self.assertEqual(nd["a"]["e"], "f")
        self.assertEqual(nd["a"]["b"]["c"], "d")

    def test_attrAccess(self):
        nd = NestedDict(self.data)

        self.assertThat(nd.a, IsInstance(NestedDict))
        self.assertEqual(self.nestedToDict(nd.a.data), self.data["a"])
        self.assertEqual(nd.a.e, nd["a"]["e"])
        self.assertEqual(nd.a.b.c, "d")

    def test_setattr(self):
        nd = NestedDict()
        setattr(nd, "a.b.c", "d")

        self.assertEqual("d", nd.data["a"]["b"]["c"])

    def test_get(self):

        nd = NestedDict(self.data)

        self.assertThat(nd.get("a"), IsInstance(NestedDict))
        self.assertEqual(self.nestedToDict(nd.get("a").data), self.data["a"])
        self.assertEqual(nd.get("a").get("e"), nd["a"]["e"])
        self.assertEqual(nd.a.get("b").c, "d")

        self.assertIsNone(nd.get("x"), None)
        self.assertEqual(nd.get("x", "y"), "y")

    def test_has(self):

        nd = NestedDict(self.data)

        self.assertTrue(nd.has("a"))
        self.assertFalse(nd.has("b"))
        self.assertFalse(nd.has("parent"))

    def test_call(self):

        nd = NestedDict(self.data)

        self.assertEqual(self.data, self.nestedToDict(nd()))
        self.assertEqual(self.data["a"], self.nestedToDict(nd.a()))

    def test_update(self):

        nd = NestedDict({})
        nd.update(self.data)
        nd.update({"a": {"b": {"i": "j"}}})
        nd.update({"a": {"b": {"c": "x"}}})
        nd.update({"a": {"g": "h"}})

        self.assertTrue(nd.a.has("g"))
        self.assertTrue(nd.a.b.has("c"))
        self.assertEqual(nd.a.b.c, "x")
        self.assertTrue(nd.a.b.has("i"))
        self.assertEqual(nd.a.b.i, "j")
        self.assertEqual(nd.a.g, "h")
        self.assertEqual(nd.a.e, "f")

    def test_lookup(self):

        nd = NestedDict(self.data)

        self.assertEqual("d", nd.lookup(("a", "b", "c")))
        self.assertIsNone(None, nd.lookup(("a", "x", "y")))
        self.assertEqual(self.data["a"], self.nestedToDict(nd.lookup(["a"])))

    def test_items(self):

        nd = NestedDict(self.data)

        for k, v in nd.items():
            self.assertIn(k, self.data)
            self.assertEqual(self.data[k], self.nestedToDict(v))

    def test_parent(self):

        nd = NestedDict(self.data)
        self.assertEqual(nd, nd.a.parent)
        self.assertIsNone(nd.parent)

        nd.update({"parent": "foo"})
        self.assertIsNone(nd.parent)
        self.assertEqual("foo", nd.get("parent"))

    def test_in(self):

        nd = NestedDict(self.data)
        self.assertIn("a", nd)
        self.assertNotIn("g", nd)

    def test_delete(self):

        nd = NestedDict(self.data)

        self.assertIn("b", nd["a"])
        delattr(nd["a"], "b")
        self.assertNotIn("b", nd["a"])

        self.assertIn("e", nd["a"])
        del nd["a"]["e"]
        self.assertNotIn("e", nd["a"])

        self.assertIn("a", nd)
        nd.delete("a")
        self.assertNotIn("a", nd)

    def test_length(self):

        nd = NestedDict(self.data)

        self.assertEqual(len(nd), len(self.data))

    def test_repr(self):

        nd = NestedDict()

        self.assertEqual(repr(nd), "<NestedDict ({})>")

        nd.update(self.data)

        self.assertEqual(repr(nd), "<NestedDict (%s)>" % repr(nd.data))

    def test_keys(self):

        nd = NestedDict(self.data)
        keys = nd.keys()

        for key in keys:
            self.assertIn(key, self.data)

    def test_values(self):

        nd = NestedDict(self.data)

        self.assertEqual(list(self.data.values()), [self.nestedToDict(x) for x in nd.values()])

    def test_haskey(self):

        nd = NestedDict(self.data)

        for key in self.data.keys():
            self.assertTrue(nd.has_key(key))

    def test_contains(self):

        nd = NestedDict(self.data)

        for key in self.data.keys():
            self.assertTrue(key in nd)

    def test_mapping(self):

        nd = NestedDict(self.data)

        d = {}
        for key, value in self.data.items():
            d[key] = nd[key]

        self.assertEqual(dict(nd), dict(**nd))
        self.assertEqual(dict(nd), d)


    def test_hasattr(self):
        nd = NestedDict(self.data)

        self.assertTrue(hasattr(nd, "a"))
        self.assertTrue(hasattr(nd, "a.e"))
        self.assertTrue(hasattr(nd, "a.b.c"))
        self.assertFalse(hasattr(nd, "b"))
        self.assertFalse(hasattr(nd, "a.b.c.d"))

def test_suite():
    from unittest import TestLoader
    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2012, Christian Kampka
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


class NestedDict(object):

    def __init__(self, dict={}):
        object.__setattr__(self, "parent", None)
        object.__setattr__(self, "data", {})
        self.update(dict)

    def __getitem__(self, key):
        rv = self.data[key]
        if type(rv) is type(self) and not rv.parent:
            object.__setattr__(rv, "parent", self)
            return rv
        return rv

    def __getattr__(self, key):
        keys = key.split(".")
        try:
            r = self[keys[0]]
            if not len(keys[1:]):
                return r
            if type(r) not in (dict, NestedDict):
                raise AttributeError("No attribute found for %s" % keys[1:])

            return getattr(r, ".".join(keys[1:]))
        except KeyError as e:
            raise AttributeError(e)

    def get(self, key, default=None):
        try:
            return self[key]
        except (KeyError, IndexError):
            return default

    def __setattr__(self, key, value):
        r = d = {}
        keys = key.split(".")
        for k in keys[:-1]:
            d = d.setdefault(k, {})
        d[keys[-1]] = value
        self.update(r)

    def has(self, key):
        return key in self.data

    def __call__(self):
        return self.data

    def update(self, other):
        if type(other) in (dict, NestedDict):
            for (key, value) in other.items():
                if key in self.data and \
                    type(self[key]) in (dict, NestedDict) and \
                        type(value) in (dict, NestedDict):
                    self.data[key] = NestedDict(self[key])
                    object.__setattr__(self.data[key], "parent", self)
                    self.data[key].update(value)
                else:
                    if type(value) is dict:
                        self.data[key] = NestedDict(value)
                    else:
                        self.data[key] = value
        else:
            self.data.update(other)

    def lookup(self, path, default=None):
        b = self.data.copy()
        for i in path[:-1]:
            b = b.get(i, {})
        return b.get(path[-1], default)

    def __iter__(self):
        return self.data.__iter__()

    def items(self):
        return self.data.items()

    def delete(self, key):
        del self.data[key]

    def __delitem__(self, key):
        self.delete(key)

    def __delattr__(self, name):
        self.delete(name)

    def __len__(self):
        return len(self.data)

    def __repr__(self):
        return "<NestedDict (%s)>" % repr(dict(self.items()))

    def keys(self):
        return self.data.keys()

    def values(self):
        return self.data.values()

    def has_key(self, key):
        return self.has(key)

    def __contains__(self, key):
        return self.has(key)


__all__ = ["NestedDict"]



# Copyright (c) 2012, Christian Kampka
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from argparse import ArgumentParser
from testtools import TestCase
from yconf.util import NestedDict


class ArgumentParserTest(TestCase):
    """
    Make sure NestedDict can be used as an argparse namespace properly.
    """

    def test_storeAction(self):

        parser = ArgumentParser()
        parser.add_argument("--test", dest="a.b")
        ns = parser.parse_args(["--test", "c"], namespace=NestedDict())

        self.assertEqual("c", ns.a.b)

    def test_default(self):
        parser = ArgumentParser()
        parser.add_argument("--test", dest="a.b", default="c", action='store')
        ns = parser.parse_args([], namespace=NestedDict())

        self.assertEqual("c", ns.a.b)

    def test_storeCons(self):
        # taken from argparse docs
        parser = ArgumentParser()
        parser.add_argument('integers', metavar='N', type=int, nargs='+')
        parser.add_argument('--sum', dest='a.b', action='store_const',
                           const=sum, default=max)
        ns = parser.parse_args(["1", "2", "3", "4", "--sum"], namespace=NestedDict())

        self.assertEqual(10, ns.a.b(ns.integers))


def test_suite():
    from unittest import TestLoader
    return TestLoader().loadTestsFromName(__name__)



# Copyright (c) 2012, Christian Kampka
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from setuptools import setup, find_packages

def readme():
        with open("README.rst") as f:
                    return f.read()

setup(
    name='yconf',
    version="0.5",
    license='BSD-3',
    author='Christian Kampka',
    author_email='christian@kampka.net',
    description='Wrapper that combines argparse with yaml config files.',
    long_description=readme(),
    url='https://github.com/kampka/yconf',
    install_requires=[
        "PyYAML >= 3.1",
    ],
    tests_require=[
        "fixtures",
        "testtools"
    ],
    test_suite="yconf.tests",
    packages=find_packages(),
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
        ],
    )
